#include <stdint.h>
#include <assert.h>
#include "flash_api.h"
#include "minmax.h"
#include "backend.h"
#include "bench.h"

bool flash_read_data(struct backend *b, uint32_t addr, uint8_t *data, uint16_t len) {
    assert(b);
    assert(data);

    uint32_t ns = 
        5 * T_SPI_BYTE_NS + // set address
        len * MAX(T_READ_BYTE_NS, T_SPI_BYTE_NS);

    bench_add_io_time(b->bench, ns);

    if (!backend_read(b, addr, data, len)) {
        return false;
    }
    return true;
}

/* these return a status code, 0 = OK */

bool flash_write_data(struct backend *b, uint32_t addr, uint8_t *data, uint16_t len) {
    assert(b);
    assert(data);

    uint32_t ns = 
        1 * T_SPI_BYTE_NS + // enable writes
        5 * T_SPI_BYTE_NS + // set address
        len * T_SPI_BYTE_NS + // send data
        // data is being written - wait for status flag (2-byte polling)
        MAX(2 * T_SPI_BYTE_NS, len * T_WRITE_BYTE_NS);

    // XXX unclear if programming happens concurrently with data loading or not        
    bench_add_io_time(b->bench, ns);
    bench_block_written(b->bench, addr / BLOCK_SIZE);

    if (!backend_write(b, addr, data, len)) {
        return false;
    }
    return true;
}

bool flash_erase_block(struct backend *b, uint32_t addr) {
    assert(b);

    uint64_t ns =
        1 * T_SPI_BYTE_NS + // erase command
        5 * T_SPI_BYTE_NS + // set address
        // wait for completion (2-byte polling)
        MAX(2 * T_SPI_BYTE_NS, T_BLOCK_ERASE_NS);

    bench_add_io_time(b->bench, ns);
    bench_block_erased(b->bench, addr / BLOCK_SIZE);

    if (!backend_erase(b, addr, BLOCK_SIZE)) {
        return false;
    }
    return true;
}

bool flash_erase_chip(struct backend *b) {
    assert(b);

    uint64_t ns =
        1ull * T_SPI_BYTE_NS + // erase command
        // wait for completion (2-byte polling)
        MAX(2ull * T_SPI_BYTE_NS, T_MASS_ERASE_NS);

    bench_add_io_time(b->bench, ns);

    for(int i = 0; i < CHIP_SIZE / BLOCK_SIZE; i++) {        
        bench_block_erased(b->bench, i);
    }

    backend_wipe(b);
    return true;
}
