// Simulated SPI FLASH API
// The bench is attached to the backed passed to each of the functions.

#ifndef FSIM_FLASH_API_H
#define FSIM_FLASH_API_H

#include <stdint.h>
#include <stdbool.h>
#include "backend.h"

#define CHIP_SIZE (64ULL * 1024ULL * 1024ULL) // 64 MiB
#define BLOCK_SIZE (256ULL * 1024ULL) // 256 KiB
#define PAGE_SIZE 512ULL // 512 B
#define BLOCK_COUNT (CHIP_SIZE/BLOCK_SIZE)

#define T_SPI_BYTE_NS    500ULL
#define T_ERASE_BYTE_NS 2000ULL
#define T_WRITE_BYTE_NS  667ULL
#define T_READ_BYTE_NS   160ULL

// Read rate 6.25 MBps
// Program rate 1.5MBps (1500kBps)
// Erase rate 0.5MBps (500kBps)

#define T_BLOCK_ERASE_NS (BLOCK_SIZE * T_ERASE_BYTE_NS)
#define T_PAGE_WRITE_NS (PAGE_SIZE * T_WRITE_BYTE_NS)

// XXX not specified in the datasheet
#define T_MASS_ERASE_NS (CHIP_SIZE * T_ERASE_BYTE_NS)

bool __attribute__((warn_unused_result)) flash_read_data(struct backend *b, uint32_t addr, uint8_t *data, uint16_t len);

bool __attribute__((warn_unused_result)) flash_write_data(struct backend *b, uint32_t addr, uint8_t *data, uint16_t len);

bool __attribute__((warn_unused_result)) flash_erase_block(struct backend *b, uint32_t addr);

bool __attribute__((warn_unused_result)) flash_erase_chip(struct backend *b);

#endif /* FSIM_FLASH_API_H */
